/* 
*	Copyright (C) 2006-2011 Team MediaPortal
*	http://www.team-mediaportal.com
*
*  This Program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2, or (at your option)
*  any later version.
*   
*  This Program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*  GNU General Public License for more details.
*   
*  You should have received a copy of the GNU General Public License
*  along with GNU Make; see the file COPYING.  If not, write to
*  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
*  http://www.gnu.org/copyleft/gpl.html
*
*/

// mpiptvsource.cpp : Defines the exported functions for the DLL application.
//
#pragma warning(disable : 4996)
#include "mpiptvsource.h"
#include <windows.h>
#include <Ws2tcpip.h>
#include <winbase.h>
#include <stdio.h>
#include <shlobj.h>

#ifdef LOGGING
static char logFile[MAX_PATH];
static WORD logFileParsed = -1;

void GetLogFile(char *pLog)
{
  SYSTEMTIME systemTime;
  GetLocalTime(&systemTime);
  if(logFileParsed != systemTime.wDay)
  {
    TCHAR folder[MAX_PATH];
    ::SHGetSpecialFolderPath(NULL,folder,CSIDL_COMMON_APPDATA,FALSE);
    sprintf(logFile,"%s\\Team MediaPortal\\MediaPortal TV Server\\log\\MPIPTVSource-%04.4d-%02.2d-%02.2d.Log",folder, systemTime.wYear, systemTime.wMonth, systemTime.wDay);
    logFileParsed=systemTime.wDay; // rec
  }
  strcpy(pLog, &logFile[0]);
}

void LogDebug(const char *fmt, ...)
{
  va_list ap;
  va_start(ap,fmt);

  char buffer[1000];
  int tmp;
  va_start(ap,fmt);
  tmp=vsprintf(buffer, fmt, ap);
  va_end(ap);
  SYSTEMTIME systemTime;
  GetLocalTime(&systemTime);

  TCHAR filename[1024];
  GetLogFile(filename);
  FILE* fp = fopen(filename,"a+");

  if (fp!=NULL)
  {
    fprintf(fp,"%02.2d-%02.2d-%04.4d %02.2d:%02.2d:%02.2d.%03.3d [%x]%s\n",
      systemTime.wDay, systemTime.wMonth, systemTime.wYear,
      systemTime.wHour,systemTime.wMinute,systemTime.wSecond,
      systemTime.wMilliseconds,
      GetCurrentThreadId(),
      buffer);
    fclose(fp);
  }
  char buf[1000];
  sprintf(buf,"%02.2d-%02.2d-%04.4d %02.2d:%02.2d:%02.2d %s\n",
    systemTime.wDay, systemTime.wMonth, systemTime.wYear,
    systemTime.wHour,systemTime.wMinute,systemTime.wSecond,
    buffer);
  ::OutputDebugString(buf);
};

void writeStream(char *data, int dataLen) {
	FILE * pFile;
//	pFile = fopen ("C:\\testvideo.log","a");
//	fwrite(data, dataLen, 1, pFile);
//	for (int i=0; i<dataLen; i++)
//	fputc ( (int) data[2] , pFile );
//	fputc ( (int) data[3] , pFile );
//	fclose (pFile);
	pFile = fopen ("C:\\testvideo2.ts","a");
	fwrite(data, dataLen, 1, pFile);
	fclose (pFile);
};

#endif

MPIptvSourceStream::MPIptvSourceStream(HRESULT *phr, CSource *pFilter)
  : CSourceStream(NAME("MediaPortal IPTV Stream"), phr, pFilter, L"Out"),
  ip(NULL),
  username(NULL),
  get_string(NULL),
  protocol(NULL),
  port(0) ,
  transport(0),
  m_socket(-1),
  m_seqNumber(0),
  hData(NULL),
  hConnection(NULL),
  hINet(NULL)
{
}

MPIptvSourceStream::~MPIptvSourceStream()
{
  Clear();
}

void MPIptvSourceStream::Clear() 
{
  switch (transport) {
    case TUDP:
    case TRTP:
      if(m_socket >= 0) {closesocket(m_socket); m_socket = -1;};
      break;
    case THTTP:
      InternetCloseHandle(hData);
      hData = NULL;
      InternetCloseHandle(hConnection);
      hConnection = NULL;
      InternetCloseHandle(hINet);
      hINet = NULL;
      break;
  }

  if(CAMThread::ThreadExists())
  {
    CAMThread::CallWorker(CMD_EXIT);
    CAMThread::Close();
  }
  if (ip) 
  {
    CoTaskMemFree(ip);
    ip = NULL;
  }
  if (username) 
  {
    CoTaskMemFree(username);
    username = NULL;
  }
  if (get_string) 
  {
    CoTaskMemFree(get_string);
	  get_string = NULL;
  }
  if (protocol)
  {
    CoTaskMemFree(protocol);
    protocol = NULL;
  }
}

HRESULT MPIptvSourceStream::GetMediaType(__inout CMediaType *pMediaType) 
{
  pMediaType->majortype = MEDIATYPE_Stream;
  pMediaType->subtype = MEDIASUBTYPE_MPEG2_TRANSPORT;

  return S_OK;
}

HRESULT MPIptvSourceStream::DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *pRequest) 
{
  HRESULT hr;
  CAutoLock cAutoLock(m_pFilter->pStateLock());

  CheckPointer(pAlloc, E_POINTER);
  CheckPointer(pRequest, E_POINTER);

  // Ensure a minimum number of buffers
  if (pRequest->cBuffers == 0)
  {
    pRequest->cBuffers = 1;
  }
  pRequest->cbBuffer = IPTV_BUFFER_SIZE;

  ALLOCATOR_PROPERTIES Actual;
  hr = pAlloc->SetProperties(pRequest, &Actual);
  if (FAILED(hr)) 
  {
    return hr;
  }

  // Is this allocator unsuitable?
  if (Actual.cbBuffer < pRequest->cbBuffer) 
  {
    return E_FAIL;
  }

  return S_OK;
}

HRESULT MPIptvSourceStream::OnThreadCreate(void) 
{
 return NOERROR;
}

HRESULT MPIptvSourceStream::OnThreadStartPlay(void) 
{
#ifdef LOGGING
  LogDebug("Entering OnThreadCreate");
#endif
  unsigned long nonblocking = 1;
  switch (transport) 
  {
  case TUDP:
  case TRTP:
    WSADATA wsaData;
    WSAStartup(MAKEWORD(2, 2), &wsaData);

    memset(&addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    if (username) 
    {
      addr.sin_addr.s_addr = inet_addr(username);
    } 
    else 
    {
      addr.sin_addr.s_addr = htonl(INADDR_ANY);
    }
    addr.sin_port = htons((u_short)port);

    IP_MREQ imr; 
    imr.imr_multiaddr.s_addr = inet_addr(ip);
    if (username) 
    {
      imr.imr_interface.s_addr = inet_addr(username);
    } 
    else 
    {
      imr.imr_interface.s_addr = INADDR_ANY;
    }

    if((m_socket = socket(AF_INET, SOCK_DGRAM, 0)) >= 0)
    {
      DWORD dw = TRUE;
      int dwLen = sizeof(dw);
      if(setsockopt(m_socket, SOL_SOCKET, SO_REUSEADDR, (const char*)&dw, sizeof(dw)) < 0)
      {
        closesocket(m_socket);
        m_socket = -1;
        return E_FAIL;
      }

      if(setsockopt(m_socket, SOL_SOCKET, SO_BROADCAST, (const char*)&dw, sizeof(dw)) < 0)
      {
        closesocket(m_socket);
        m_socket = -1;
        return E_FAIL;
      }

      getsockopt(m_socket, SOL_SOCKET, SO_RCVBUF, (char *)&dw, &dwLen);
#ifdef LOGGING
      LogDebug("Socket receive buffer is: %d (%d)", dw, dwLen);

      LogDebug("Trying to set receive buffer to %d", IPTV_SOCKET_BUFFER_SIZE);
#endif
      dw = IPTV_SOCKET_BUFFER_SIZE;
      if(setsockopt(m_socket, SOL_SOCKET, SO_RCVBUF, (const char*)&dw, sizeof(dw)) < 0)
      {
        closesocket(m_socket);
        m_socket = -1;
        return E_FAIL;
      }

      dwLen = sizeof(dw);
      getsockopt(m_socket, SOL_SOCKET, SO_RCVBUF, (char *)&dw, &dwLen);
#ifdef LOGGING
      LogDebug("New socket receive buffer is: %d (%d)", dw, dwLen);
#endif
      if (ioctlsocket(m_socket, FIONBIO, &nonblocking) != 0) 
      {
        closesocket(m_socket);
        m_socket = -1;
        return E_FAIL;
      }

      if(bind(m_socket, (struct sockaddr*)&addr, sizeof(addr)) < 0)
      {
        closesocket(m_socket);
        m_socket = -1;
      }

      if(IN_MULTICAST(htonl(imr.imr_multiaddr.s_addr)))
      {
        int ret = setsockopt(m_socket, IPPROTO_IP, IP_ADD_MEMBERSHIP, (const char*)&imr, sizeof(imr));
        if(ret < 0) ret = ::WSAGetLastError();
        ret = ret;
      }


      networkBuffer = NULL;
      unusedBuffer = NULL;

      this->packetSize = -1;

      // Create a new event
      winsockEvent[0] = WSACreateEvent();


      // Associate event type FD_READ
      // with the socket and new event
      if (WSAEventSelect(m_socket, winsockEvent[0], FD_READ) != 0) 
      {
        int errorcode = WSAGetLastError();
        closesocket(m_socket);
        m_socket = -1;
      }

      // Initialize internal buffer
      this->dataBuffer = new char[IPTV_INTERNAL_INITIAL_BUFFER_SIZE];
      this->currentBufferSize = IPTV_INTERNAL_INITIAL_BUFFER_SIZE;
      this->dataStart = this->dataBuffer;
      this->currentBufferUsed = 0;


      InitializeCriticalSection(&criticalSection);
      // start winsock worker thread
      this->threadShouldExit = false;
      hWinsockWorkerThread = CreateThread( 
        NULL,                   // default security attributes
        0,                      // use default stack size  
        &MPIptvSourceStream::WinsockWorker,       // thread function name
        this,          // argument to thread function 
        0,                      // use default creation flags 
        &dwWinsockWorkerThreadId);   // returns the thread identifier 
        SetThreadPriority(hWinsockWorkerThread, THREAD_PRIORITY_HIGHEST);

    } 
    else
      return E_FAIL;
    break;
  case THTTP: 
#ifdef LOGGING
    LogDebug("HTTP - IP:%s Port:%d Get string:'%s'",ip,port,get_string);
#endif
    hINet = InternetOpen("Mozilla/4.0", INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0 );
    if ( hINet ) 
      hConnection = InternetConnect( hINet, ip, port, " ", " ", INTERNET_SERVICE_HTTP, 0, 0 );

    if ( hConnection ) 
    {
      hData = HttpOpenRequest( hConnection, "GET", get_string, 
        NULL, NULL, NULL, INTERNET_FLAG_KEEP_CONNECTION, 0 );
    } 
    else 
    {
      InternetCloseHandle(hINet);
    }

    if ( hData ) 
    {
		  if (!HttpSendRequest( hData, NULL, -1, NULL, 0)) 
      {
#ifdef LOGGING
			  LogDebug("Error in HttpSendRequest %f",(DWORD)GetLastError);
#endif
		  }
	  } 
    else 
    {
		  InternetCloseHandle(hINet);
		  InternetCloseHandle(hConnection);
		  return E_FAIL;
	  }
    break;
  }
  totalReceived = 0;
  rtpFailPackets = 0;

  SetThreadPriority(m_hThread, THREAD_PRIORITY_ABOVE_NORMAL);
  return NOERROR;
}

HRESULT MPIptvSourceStream::FillBuffer(IMediaSample *pSamp)
{
#ifdef EXTENSIVELOGGING
  LogDebug("Enter FillBuffer. Protokoll is %d", transport);
#endif
  CheckPointer(pSamp, E_POINTER);

  // Now get buffers address
  char *pData;
  long cbData;

  int resp; 
  //int len;
  
  if (pSamp->GetPointer((BYTE **)&pData) != S_OK) 
  {
#ifdef LOGGING
    LogDebug("Unable to get mediaSample-Pointer. Exiting with error.");
#endif
    return E_FAIL;
  }
  cbData = pSamp->GetSize();
  if (cbData <= 0) 
  {
#ifdef LOGGING
    LogDebug("Unable to get size of mediaSample buffer. Exiting with error.");
#endif
    return E_FAIL;
  }

  // Decide what to do on protocol
  switch (transport) 
  {
  case TUDP:
    {
   // len = recv(m_socket,pData, cbData, 0);
#ifdef EXTENSIVELOGGING
    LogDebug("UDP: Received bytes: %d", len);
#endif
    Sleep(50);
    int lengthOfPostableData = 0;
    EnterCriticalSection(&this->criticalSection);
    lengthOfPostableData = min(this->currentBufferUsed, cbData);
    char* bufferEnd = this->dataBuffer + this->currentBufferSize;
    char* dataEnd = this->dataStart + lengthOfPostableData;
    if (dataEnd > bufferEnd) 
    {
      lengthOfPostableData = (int)(bufferEnd - this->dataStart);
    }
    // Copy data
    memcpy(pData, this->dataStart, lengthOfPostableData);
    
    this->dataStart += lengthOfPostableData;
    this->currentBufferUsed -= lengthOfPostableData;
     if (this->dataStart == bufferEnd)
       this->dataStart = this->dataBuffer;
    LeaveCriticalSection(&this->criticalSection);

    pSamp->SetActualDataLength(lengthOfPostableData);
    totalReceived += lengthOfPostableData;
    // On an udp stream we cannot see if there was a packet lost, nor if this is a synchron point nor the timestamps. 
    // So, just leave it like it is and let the other filter decide
#ifdef EXTENSIVELOGGING
    LogDebug("UDP: Reached end of FillBuffer. Total bytes received: %d", totalReceived);
#endif
    return S_OK;
    }
  case THTTP: 
    {
    int len = 0;
    resp = InternetReadFile( hData, pData, cbData, (LPDWORD)&len ) ;
#ifdef EXTENSIVELOGGING
    LogDebug("HTTP - Received %d bytes. Response %d Ordered bytes %d", len, resp, cbData);
#endif
    
    if (!resp) 
    {
#ifdef LOGGING
			LogDebug("False responce 0x%x",GetLastError());
#endif
      return E_FAIL;
		}
    // If nothing recieved wait 100ms and read once more
    if (len == 0) 
    {
      Sleep(100);  
      resp = InternetReadFile( hData, pData, cbData, (LPDWORD)&len ) ;
#ifdef EXTENSIVELOGGING
      LogDebug("HTTP 2nd try - Received %d bytes. Response %d", len, resp);
#endif
      
      if (!resp) 
      {
#ifdef LOGGING
			  LogDebug("False responce 0x%x",GetLastError());
#endif
        return E_FAIL;
		  }
    }
    totalReceived += len;
    pSamp->SetActualDataLength(len);
    return S_OK;
    }
  case TRTP:
    // This is an rtp stream

    // Wait until the buffer is filled up
    if (!rtpHandler.CheckDataCanBePosted(cbData)) 
    {
      Sleep(50);
      pSamp->SetActualDataLength(0);
    } 
    else 
    {
      // Post data
      // Now get the data and fill the buffer
      unsigned int firstTimestamp;
      unsigned int lastTimestamp;
      bool isContinous;
      EnterCriticalSection(&this->criticalSection);
      int processedBytes = rtpHandler.GetPacketData(pData, cbData, firstTimestamp, lastTimestamp, isContinous);
      LeaveCriticalSection(&this->criticalSection);
      #ifdef EXTENSIVELOGGING
          LogDebug("RTP: Sending data to next filter: Len: %d, firstTS: %d, lastTS: %d, continous: %d", processedBytes, firstTimestamp, lastTimestamp, isContinous);
      #endif
      REFERENCE_TIME rtStart = 0; //firstTimestamp;
      REFERENCE_TIME rtStop  = 0; //lastTimestamp;
      pSamp->SetTime(&rtStart, &rtStop);
      pSamp->SetActualDataLength(processedBytes);
      pSamp->SetDiscontinuity(!isContinous);
#ifdef EXTENSIVELOGGING
      if (!isContinous)
        LogDebug("RTP: Got unContinous packet!");
      LogDebug("RTP: Exiting Fillbuffer");
#endif
    }
    return S_OK;
  }
  return S_OK;
}

DWORD WINAPI MPIptvSourceStream::WinsockWorker( LPVOID lpParam ) 
{
  MPIptvSourceStream* caller;
  caller = (MPIptvSourceStream*)lpParam;
  // Buffer to get (full)packet when buffer is filled up (udp)
  char* tempBuffer = NULL;

  // This buffer will hold the data only for a short time. Its nessecary because we need to skip the first 12 bytes (rtp)
  char *pRawData = new char[10000];

  while (!caller->threadShouldExit) 
  {
    // Wait for data packet to arrive
    DWORD res = WSAWaitForMultipleEvents(1, 
      caller->winsockEvent, 
      FALSE, 
      1000, // Wait for max 1 second 
      FALSE);
    WSAResetEvent(caller->winsockEvent[0]);
    if (res != WSA_WAIT_TIMEOUT) 
    {
      // Decide to do on transport protocoll
      switch (caller->transport) 
      {
        case TRTP:
        {
            // Was there an error in the stream?
            bool failPacket = false;
              // Get packet
              int len = recv(caller->m_socket,pRawData, 10000, 0);
              while (len > 0) {
#ifdef EXTENSIVELOGGING
                LogDebug("RTP: Received bytes: %d", len);
#endif
                // Check if we got some data
                EnterCriticalSection(&caller->criticalSection);
                  // Do some basic checks to see if this is really a rtp packet
                  if (!caller->rtpHandler.IsRTPpacket(pRawData, len)) 
                  {
                    // This is not a rtp packet. If there are RTP_MAX_FAIL_PACKETS none rtp packets in a row, consider this stream as udp instead
                    caller->rtpFailPackets++;
  #ifdef LOGGING
                    LogDebug("RTP: Invalid rtp packet received. FailPackets: %d", RTPfailPackets);
  #endif
                    if (caller->rtpFailPackets >= RTP_MAX_FAIL_PACKETS) 
                    {
                      // This is not a rtp stream! Consider it as raw udp.
                      caller->transport = TUDP;
  #ifdef LOGGING
                      LogDebug("RTP: Switching back to raw udp");
  #endif
                      // Skip this packet
                      continue;
                    }
                    // This is still not a correct rtp packet -> skip it
                    continue;
                  } 
                  else
                    caller->rtpFailPackets = 0;

                  // Now we can be sure, that this is a rtp packet -> process it
                  if (!caller->rtpHandler.ProcessPacket(pRawData, len))
                    return E_FAIL;
                  caller->totalReceived += len;
                  LeaveCriticalSection(&caller->criticalSection);
                  len = recv(caller->m_socket,pRawData, 10000, 0);
              }
              
            break;
        }

        case TUDP:
          EnterCriticalSection(&caller->criticalSection);
          //LogDebug("WorkerThread: currentBuffersize:%d, currentUsed:%d, packetSize:%d", caller->currentBufferSize, caller->currentBufferUsed, caller->packetSize);
          // Check if buffer needs to be extended
          if ((caller->currentBufferSize - caller->currentBufferUsed) < caller->packetSize) 
          {
            // Check if max buffer size is reached
            if ((caller->currentBufferSize*2) <= IPTV_INTERNAL_MAX_BUFFER_SIZE) 
            {
              // Create new buffer
              caller->currentBufferSize *= 2;
              char* newBuffer = new char[caller->currentBufferSize];
              // Copy data into new buffer
              long dataStartOffset = (long)caller->dataStart - (long)caller->dataBuffer;
              if ((dataStartOffset + caller->currentBufferUsed) <= (caller->currentBufferSize/2)) 
              {
                // Copy data
                memcpy(newBuffer, caller->dataStart, caller->currentBufferUsed);
              } 
              else 
              {
                // Copy first part
                long firstPartLen = caller->currentBufferSize/2 - dataStartOffset;
                memcpy(newBuffer, caller->dataStart, firstPartLen);
                
                // Pointer arithmetic
                char *secondPartStart = newBuffer + firstPartLen;
                // Copy second part
                memcpy(secondPartStart, caller->dataBuffer, caller->currentBufferUsed-firstPartLen);
              }
              // delete old buffer
              delete[] caller->dataBuffer;
              caller->dataBuffer = newBuffer;
              caller->dataStart = caller->dataBuffer;
            }
          }

          long endOfBuffer = (long)caller->dataBuffer + caller->currentBufferSize;
          long endOfUsedBuffer = (long)caller->dataStart + caller->currentBufferUsed;
          if (endOfUsedBuffer >= endOfBuffer)
            endOfUsedBuffer -= caller->currentBufferSize;
          long sizeAtEndOfBuffer = endOfBuffer - endOfUsedBuffer;

          if (sizeAtEndOfBuffer >= caller->packetSize) 
          {
            int len = recv(caller->m_socket,(char*)endOfUsedBuffer, sizeAtEndOfBuffer, 0);
            if (len > caller->packetSize) 
            {
              if (tempBuffer) delete[] tempBuffer;
              caller->packetSize = len;
              tempBuffer = new char[len];
            }
            if (len > 0) 
            {
              caller->currentBufferUsed += len;
            } 
          } 
          else if (sizeAtEndOfBuffer >= 0) 
          {
            int len = 0;
            if (sizeAtEndOfBuffer > 0) 
            {
              len = recv(caller->m_socket,tempBuffer, caller->packetSize, 0);
              if (len > 0) 
              {
                // Copy first part into buffer
                int copyLength = min(sizeAtEndOfBuffer, len);
                memcpy((char*)endOfUsedBuffer, tempBuffer, copyLength);
                caller->currentBufferUsed += copyLength;
                len -= copyLength;
                if (len > 0) 
                {
                  memcpy(caller->dataBuffer, tempBuffer + copyLength, len);
                  caller->currentBufferUsed += len;
                }
              }
            }
          } 
          else 
          {
            // Error!! This shouldn't happen at all
            return E_FAIL;
          }
          LeaveCriticalSection(&caller->criticalSection);
          break;
      }
    }
  }
  return S_OK;
}

HRESULT MPIptvSourceStream::OnThreadDestroy(void) 
{
  switch (transport) 
  {
    case TUDP:
    case TRTP:
      this->threadShouldExit = false;
      if(m_socket >= 0) {closesocket(m_socket); m_socket = -1;};
      WSACleanup();
      delete[] this->dataBuffer;
      this->dataBuffer = NULL;
      this->dataStart = NULL;
      // Wait for the winsock worker thread to exit      
      if (WaitForSingleObject(this->hWinsockWorkerThread, 1000) == WAIT_TIMEOUT)
        // Thread didn't exit. Kill it now
        TerminateThread(this->hWinsockWorkerThread, 0);
      DeleteCriticalSection(&criticalSection);
      break;
    case THTTP:
      InternetCloseHandle(hData);
      InternetCloseHandle(hConnection);
      InternetCloseHandle(hINet);
      break;
  }

#ifdef LOGGING
  LogDebug("Total read bytes %d", totalReceived); 
#endif

  return NOERROR;
}

void ZeroURL(URL_COMPONENTS *url) 
{
  url->lpszScheme = NULL;
  url->lpszExtraInfo = NULL;
  url->lpszHostName = NULL;
  url->lpszPassword = NULL;
  url->lpszUrlPath = NULL;
  url->lpszUserName = NULL;

  url->dwSchemeLength    = -1;
  url->dwHostNameLength  = -1;
  url->dwUrlPathLength   = -1;
  url->dwExtraInfoLength = -1;
};

typedef bool (__cdecl * pICFUNC)(URL_COMPONENTS url, char* buffer); 
#define DLLMAX 256
extern int dllTotal;
extern struct PreauthLib 
{
  char* protocol;
  int transport;
  pICFUNC GetRealURL;
} dllPreAuth[DLLMAX];

bool MPIptvSourceStream::Load(const TCHAR* fn) 
{
  Clear();
  URL_COMPONENTS url;
  ZeroURL(&url);
  url.dwStructSize = sizeof(URL_COMPONENTS);
#ifdef LOGGING
  LogDebug("CMPIptvSourceStream::Load fn: [%s]",fn);
#endif
  if (!InternetCrackUrl(fn, 0, 0, &url)) 
  {
    return false;
  }
  char pBuff[2048];
  memset(pBuff,0,2048);
  memcpy(pBuff,fn,strlen(fn)+1);
  int i=0;
  while (i<dllTotal) 
  {
#ifdef LOGGING
    LogDebug("%d %s %d",dllPreAuth[i].transport, dllPreAuth[i].protocol, dllPreAuth[i].GetRealURL);
#endif
    if (strnicmp(url.lpszScheme,dllPreAuth[i].protocol,url.dwSchemeLength) == 0) 
    {
      transport = dllPreAuth[i].transport;
      if (dllPreAuth[i].GetRealURL != NULL) 
      {
        dllPreAuth[i].GetRealURL(url, pBuff);
      }
      break;
    }
    i++;
  }
  if (i == dllTotal) 
  {
#ifdef LOGGING
    LogDebug("Unsupported protocol %s", url.lpszScheme);
#endif
    return false;
  }

#ifdef LOGGING
  LogDebug("Transport %d URL %s", transport, pBuff);
#endif
  ZeroURL(&url);
  url.dwStructSize = sizeof(URL_COMPONENTS);
  if (!InternetCrackUrl(pBuff, 0, 0, &url)) 
  {
    return false;
  }
#ifdef LOGGING
  LogDebug("CMPIptvSourceStream::Load url.dwStructSize: %d url.dwSchemeLength: %d url.dwHostNameLength: %d url.dwUserNameLength : %d url.dwUrlPathLength: %d url.dwExtraInfoLength: %d", 
    url.dwStructSize, url.dwSchemeLength, url.dwHostNameLength, url.dwUserNameLength, url.dwUrlPathLength, url.dwExtraInfoLength);
#endif
  if (!(protocol = (TCHAR*) CoTaskMemAlloc((url.dwSchemeLength + 1) * sizeof(TCHAR)))) 
  {
    return false;
  }
  memset(protocol, 0, (url.dwSchemeLength + 1) * sizeof(TCHAR));
  strncat(protocol, url.lpszScheme, url.dwSchemeLength);

  if (!(ip = (TCHAR*) CoTaskMemAlloc((url.dwHostNameLength + 1) * sizeof(TCHAR)))) 
  {
    return false;
  }
  memset(ip, 0, (url.dwHostNameLength + 1) * sizeof(TCHAR));
  strncat(ip, url.lpszHostName, url.dwHostNameLength);
  if (url.dwUserNameLength > 0) 
  {
    if (!(username = (TCHAR*) CoTaskMemAlloc((url.dwUserNameLength + 1) * sizeof(TCHAR)))) 
    {
      return false;
    }
    memset(username, 0, (url.dwUserNameLength + 1) * sizeof(TCHAR));
    strncat(username, url.lpszUserName, url.dwUserNameLength);
  }

  if (!(get_string = (TCHAR*) CoTaskMemAlloc((url.dwUrlPathLength + url.dwExtraInfoLength + 1) * sizeof(TCHAR))))
  {
	  return false;
  }
  memset(get_string, 0, (url.dwUrlPathLength + url.dwExtraInfoLength + 1) * sizeof(TCHAR));
  strncat(get_string, url.lpszUrlPath, url.dwUrlPathLength );
  strncat(get_string, url.lpszExtraInfo, url.dwExtraInfoLength );
  port = url.nPort;
  return true;
}

// This is the constructor of a class that has been exported.
// see mpiptvsource.h for the class definition
MPIptvSource::MPIptvSource(IUnknown *pUnk, HRESULT *phr)
  : CSource(NAME("MediaPortal IPTV Source"), pUnk, CLSID_MPIptvSource)
{
  // The pin magically adds itself to our pin array.
  m_stream = new MPIptvSourceStream(phr, this);

  if (phr)
  {
    if (m_stream == NULL)
      *phr = E_OUTOFMEMORY;
    else
      *phr = S_OK;
  } 

  m_fn = EMPTY_STRING;
}


MPIptvSource::~MPIptvSource()
{
  delete m_stream;
  if (m_fn != EMPTY_STRING) 
  {
    CoTaskMemFree(m_fn);
    m_fn = EMPTY_STRING;
  }
}

STDMETHODIMP MPIptvSource::NonDelegatingQueryInterface(REFIID riid, void** ppv)
{
  CheckPointer(ppv, E_POINTER);

  return 
    QI(IFileSourceFilter)
    __super::NonDelegatingQueryInterface(riid, ppv);
}

STDMETHODIMP MPIptvSource::Load(LPCOLESTR pszFileName, const AM_MEDIA_TYPE* pmt) 
{
  size_t length = wcstombs(NULL, pszFileName, 0);
  if(!(m_fn = (TCHAR*)CoTaskMemAlloc((length+1)*sizeof(TCHAR))))
    return E_OUTOFMEMORY;
  wcstombs(m_fn, pszFileName, length + 1);
  if(!m_stream->Load(m_fn))
    return E_FAIL;

  return S_OK;
}

STDMETHODIMP MPIptvSource::GetCurFile(LPOLESTR* ppszFileName, AM_MEDIA_TYPE* pmt)
{
  if(!ppszFileName) return E_POINTER;

  if(!(*ppszFileName = (LPOLESTR)CoTaskMemAlloc((strlen(m_fn)+1)*sizeof(WCHAR))))
    return E_OUTOFMEMORY;

  mbstowcs(*ppszFileName, m_fn, strlen(m_fn) + 1);

  return S_OK;
}

CUnknown * WINAPI MPIptvSource::CreateInstance(IUnknown *pUnk, HRESULT *phr)
{
  MPIptvSource *pNewFilter = new MPIptvSource(pUnk, phr );

  if (phr)
  {
    if (pNewFilter == NULL) 
      *phr = E_OUTOFMEMORY;
    else
      *phr = S_OK;
  }

  return pNewFilter;
}

