/* 
 *	Copyright (C) 2006-2011 Team MediaPortal
 *	http://www.team-mediaportal.com
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#pragma once
#ifndef __MPIPTVGUIDS_DEFINED
#define __MPIPTVGUIDS_DEFINED

#define _CRT_SECURE_NO_WARNINGS
#include <winsock2.h>
#include <streams.h>
#pragma comment(lib, "wininet.lib")

#include <wininet.h>
#include "rtpsource.h"


// {D3DD4C59-D3A7-4b82-9727-7B9203EB67C0}
DEFINE_GUID(CLSID_MPIptvSource, 
0xd3dd4c59, 0xd3a7, 0x4b82, 0x97, 0x27, 0x7b, 0x92, 0x3, 0xeb, 0x67, 0xc0);

#define QI(i) (riid == __uuidof(i)) ? GetInterface((i*)this, ppv) :
//#define IN_MULTICAST(i)            (((long)(i) & 0xf0000000) == 0xe0000000)

// url format: udp://[interface]@ip:port, example: udp://192.168.1.44@233.2.3.4:1000, rtp://@233.2.3.4:1000

#define EMPTY_STRING ""
#define UDP_PROTOCOL "udp"
#define RTP_PROTOCOL "rtp"
#define IPTV_BUFFER_SIZE 10 * 1024 //By default 10KB buffer size
#define IPTV_SOCKET_BUFFER_SIZE 1024 * 1024 //Socket receive buffer size - not related to read buffer size above
//This buffer stores the data received from Winsock before it is being sent to the next filter. 
//The buffer size is a dynamic. When buffer is filled up the filter reserves more memory to sotre the data
//Initial size must be bigger than one received packet (packet is usually 1368 bytes)
#define IPTV_INTERNAL_INITIAL_BUFFER_SIZE (32 * 1024) // 32KB
// Max size of internal buffer
#define IPTV_INTERNAL_MAX_BUFFER_SIZE (3 * 1024 * 1024) // 3MB

#define RTP_MAX_FAIL_PACKETS 5  //Nr. of wrong (=wrong header) packets after the rtp-stream is considered as raw udp instead of rtp

// definitions of transport layer
#define TUDP 1
#define TRTP 2
#define THTTP 3

class MPIptvSourceStream : public CSourceStream
{

  struct NetworkPacket 
  {
    char* data;
    NetworkPacket* next;
  };

protected:

	TCHAR* protocol; // udp/rtp
	TCHAR* ip;
	WORD port;
	TCHAR* username;
	TCHAR* get_string;
  HINTERNET hINet; 
  HINTERNET hConnection;
  HINTERNET hData;
	SOCKET m_socket;

  long totalReceived;

	DWORD m_seqNumber;
  sockaddr_in addr;

	HRESULT FillBuffer(IMediaSample *pSamp);
	HRESULT GetMediaType(__inout CMediaType *pMediaType);
	HRESULT DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *pRequest);
  HRESULT OnThreadCreate(void);
  HRESULT OnThreadDestroy(void);
  HRESULT OnThreadStartPlay(void);
	bool GetUrlKartina(URL_COMPONENTS url, TCHAR* buffer);

public:

    MPIptvSourceStream(HRESULT *phr, CSource *pFilter);
    ~MPIptvSourceStream();

	bool Load(const TCHAR* fn);
	void Clear();

private:
  int transport;
  int rtpFailPackets;
  RtpSource rtpHandler;

  // Everything for the winsock worker thread
  DWORD   dwWinsockWorkerThreadId;
  HANDLE  hWinsockWorkerThread;
  bool threadShouldExit;
  static DWORD WINAPI WinsockWorker( LPVOID lpParam );
  CRITICAL_SECTION criticalSection;

  // Buffer for unprocessed network packets
  NetworkPacket* networkBuffer;
  // Pointer to store unused buffer objects (to reduce new's/delete's)
  NetworkPacket* unusedBuffer;

  // Event that is trggered when winsock gets data
  WSAEVENT winsockEvent[1];

  // Size of one data packet
  int packetSize;

  // The internal buffer
  char *dataBuffer;       // Internal buffer
  int currentBufferSize;  // Total size of buffer
  char *dataStart;        // Pointer to start of payload data within buffer
  int currentBufferUsed;  // Used bytes in buffer

};

// This class is exported from the mpiptvsource.dll
class MPIptvSource : public CSource, public IFileSourceFilter
{

private:
    // Constructor is private because you have to use CreateInstance
    MPIptvSource(IUnknown *pUnk, HRESULT *phr);
    ~MPIptvSource();

    MPIptvSourceStream *m_stream;
	TCHAR* m_fn;

public:
	// IFileSourceFilter
	DECLARE_IUNKNOWN
    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void** ppv);
	STDMETHODIMP Load(LPCOLESTR pszFileName, const AM_MEDIA_TYPE* pmt);
	STDMETHODIMP GetCurFile(LPOLESTR* ppszFileName, AM_MEDIA_TYPE* pmt);
	static CUnknown * WINAPI CreateInstance(IUnknown *pUnk, HRESULT *phr);  
};

#endif
