/* 
 *	Copyright (C) 2006-2011 Team MediaPortal
 *	http://www.team-mediaportal.com
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#pragma once
#ifndef __RTPSOURCE_DEFINED
#define __RTPSOURCE_DEFINED

#define IPTV_RTP_INTERNAL_BUFFER_SIZE 256 * 1024 //Used to reorder packages in rtp streams
#define RTP_HEADER_SIZE 12 // Size of the rtp header in byte

//#define EXTENSIVELOGGING
//#define LOGGING

// Extensive logging needs logging
#ifdef EXTENSIVELOGGING
  #define LOGGING
#endif


class RtpSource 
{
private:

  struct PacketItem 
  {
    char* data;
    int size;
    unsigned int timestamp;
    unsigned int sequenceNr;
    PacketItem* next;
  };

  
  
  unsigned int rtpSsrc;
  bool isFirstPacket;
  unsigned int lastSequenceNr;
  bool rtpStreamRunning;

  // Everything for packetItem
  PacketItem* firstItem;
  PacketItem* lastItem;
  int packetSizeTotal;
  
  // Returns true, if seuenceNr1 is the next packet after sequenceNr0
  bool SequenceIsContinuous(unsigned int sequnceNr0, unsigned int sequnceNr1);

  // Returns true, if the "item" belongs between "itemBefore" and "itemAfter"
  bool PacketBelongsBetween(PacketItem *item, PacketItem *itemBefore, PacketItem *itemAfter);

public:
  RtpSource();
  ~RtpSource();


  // Add one packed to the buffer
  void AddPacket(char* data, int length, unsigned int timestamp, unsigned int sequenceNr);
  
  // Gets packets from the buffer, put them into "buffer" and deletes them from the list.
  int GetPacketData(char* buffer, int bufferSize, unsigned int &firstTimestamp, unsigned int &lastTimestamp, bool &isContinous);

  // Check if data should be copied from the list to the mediaSample buffer
  bool CheckDataCanBePosted(int bufferSize);

  // Checks if this is a valid rtp packet (does 2 simple header checks)
  // Returns true, if it is a valid packet.
  bool IsRTPpacket(char *data, int datalen);

  // Processes the packet (cheks the sequence number and adds it to the internal buffer)
  // Returns false if something went wrong. The thread should be stopped if this function returns false!!
  bool ProcessPacket(char *data, int datalen);

};









#endif