/* 
 *	Copyright (C) 2006-2011 Team MediaPortal
 *	http://www.team-mediaportal.com
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#pragma warning(disable : 4996)
// Includes must be in this order
#include <initguid.h>
#include <cguid.h>
#include "mpiptvsource.h"
#include <streams.h>
#include "Shlwapi.h"


#define g_wszPushSource     L"MediaPortal IPTV Source Filter"

// Filter setup data
const AMOVIESETUP_MEDIATYPE iptvOpPinTypes =
{
	&MEDIATYPE_Stream,              // Major type
	&MEDIASUBTYPE_MPEG2_TRANSPORT   // Minor type
};

const AMOVIESETUP_PIN iptvOutputPin = 
{
    L"Output",      // Obsolete, not used.
    FALSE,          // Is this pin rendered?
    TRUE,           // Is it an output pin?
    FALSE,          // Can the filter create zero instances?
    FALSE,          // Does the filter create multiple instances?
    &CLSID_NULL,    // Obsolete.
    NULL,           // Obsolete.
    1,              // Number of media types.
    &iptvOpPinTypes // Pointer to media types.
};

const AMOVIESETUP_FILTER iptvPushSource =
{
    &CLSID_MPIptvSource,    // Filter CLSID
    g_wszPushSource,        // String name
    MERIT_NORMAL,           // Filter merit
    1,                      // Number pins
    &iptvOutputPin          // Pin details
};

// List of class IDs and creator functions for the class factory. This
// provides the link between the OLE entry point in the DLL and an object
// being created. The class factory will call the static CreateInstance.
// We provide a set of filters in this one DLL.

CFactoryTemplate g_Templates[1] = 
{
    { 
      g_wszPushSource,                    // Name
      &CLSID_MPIptvSource,                // CLSID
      MPIptvSource::CreateInstance,      // Method to create an instance of MyComponent
      NULL,                               // Initialization function
      &iptvPushSource                     // Set-up information (for filters)
    }
};

int g_cTemplates = sizeof(g_Templates) / sizeof(g_Templates[0]);    

#define DLLMAX 256
typedef bool (__cdecl * pICFUNC)(URL_COMPONENTS url, char* buffer); 
typedef void (__cdecl * pICFUNC1)(int* transport, char** protocol); 
pICFUNC dllGetRealURL[DLLMAX];
struct PreauthLib {
  char* protocol;
  int transport;
  pICFUNC GetRealURL;
} dllPreAuth[DLLMAX];

int dllTotal;


////////////////////////////////////////////////////////////////////////
//
// Exported entry points for registration and unregistration 
// (in this case they only call through to default implementations).
//
////////////////////////////////////////////////////////////////////////

STDAPI DllRegisterServer()
{
    return AMovieDllRegisterServer2( TRUE );
}

STDAPI DllUnregisterServer()
{
    return AMovieDllRegisterServer2( FALSE );
}

//
// DllEntryPoint
//
extern "C" BOOL WINAPI DllEntryPoint(HINSTANCE, ULONG, LPVOID);

//extern void GetLogFile1(char *pLog);
extern void LogDebug(const char *fmt, ...);
void LoadPreAuthPlugins();
void LoadDefautProtocols();

BOOL APIENTRY DllMain(HANDLE hModule, 
                      DWORD  dwReason, 
                      LPVOID lpReserved)
{
#ifdef LOGGING
  LogDebug( "DllMain - called" );
#endif
  dllTotal = 0;
  LoadDefautProtocols();
  LoadPreAuthPlugins();
  return DllEntryPoint((HINSTANCE)(hModule), dwReason, lpReserved);
}

void LoadDefautProtocols()
{
  dllPreAuth[dllTotal].GetRealURL = NULL;
  dllPreAuth[dllTotal].protocol = "udp";
  dllPreAuth[dllTotal].transport = 1; //tUDP
  dllTotal++;
  dllPreAuth[dllTotal].GetRealURL = NULL;
  dllPreAuth[dllTotal].protocol = "rtp";
  dllPreAuth[dllTotal].transport = 2; //tRTP
  dllTotal++;
  dllPreAuth[dllTotal].GetRealURL = NULL;
  dllPreAuth[dllTotal].protocol = "http";
  dllPreAuth[dllTotal].transport = 3; //tHTTP
  dllTotal++;
}

void LoadPreAuthPlugins()
{
  WIN32_FIND_DATA info;
  HANDLE h;

  LPTSTR  strDLLPath = new TCHAR[_MAX_PATH];
  LPTSTR  strDLLSearch = new TCHAR[_MAX_PATH];
  GetModuleFileName(NULL, strDLLPath, _MAX_PATH);
  PathRemoveFileSpec(strDLLPath);
  strcat(strDLLPath, "\\MPIPTVplugins\\");
  strcpy(strDLLSearch, strDLLPath);
  strcat(strDLLSearch, "*.dll");
  h = FindFirstFile(strDLLSearch, &info);
  if (h != INVALID_HANDLE_VALUE) 
  {
    do 
    {
      LPTSTR strDLLName = new TCHAR[_MAX_PATH];
      strcpy(strDLLName, strDLLPath);
      strcat(strDLLName, info.cFileName);
#ifdef LOGGING
      LogDebug("Loading %s",strDLLName);
#endif
      HINSTANCE hGetProcIDDLL = LoadLibrary(strDLLName);
#ifdef LOGGING
      LogDebug("LoadLibrary %s %d",strDLLName,hGetProcIDDLL);
#endif
      if (hGetProcIDDLL == NULL) continue;
      FARPROC lpfnGetProcessID = GetProcAddress(HMODULE(hGetProcIDDLL),"GetRealURL"); 
#ifdef LOGGING
      LogDebug("GetProcAddress %s %d",strDLLName,lpfnGetProcessID);
#endif
      if (lpfnGetProcessID == NULL) continue;
//      dllGetRealURL[dllTotal++] = pICFUNC(lpfnGetProcessID); 
      dllPreAuth[dllTotal].GetRealURL = pICFUNC(lpfnGetProcessID);

      FARPROC lpfnGetProcessID1 = GetProcAddress(HMODULE(hGetProcIDDLL),"GetProtocol"); 
#ifdef LOGGING
      LogDebug("GetProcAddress %s %d",strDLLName,lpfnGetProcessID1);
#endif
      if (lpfnGetProcessID1 == NULL) continue;
      pICFUNC1 GetProtocol;
      GetProtocol = pICFUNC1(lpfnGetProcessID1);
      GetProtocol(&dllPreAuth[dllTotal].transport,&dllPreAuth[dllTotal].protocol);
#ifdef LOGGING
      LogDebug("Found transport %d Protocol %s",dllPreAuth[dllTotal].transport, dllPreAuth[dllTotal].protocol);
#endif
      dllTotal++;
      delete strDLLName;
      if (dllTotal == DLLMAX) break;
    } while (FindNextFile(h, &info));
    FindClose(h);
#ifdef LOGGING
    LogDebug("Found %d lib(s)",dllTotal);
#endif  
  } 
  else 
  {
#ifdef LOGGING
    LogDebug("Did not find anything (%s)",strDLLPath);
#endif
  }
  delete strDLLPath;
  delete strDLLSearch;
}
